@@ -1,20 +1,26 @@
 """
-A package for generating various graphs in networkx. 
+A package for generating various graphs in networkx.
 
 """
+from networkx.generators.atlas import *
 from networkx.generators.classic import *
+from networkx.generators.community import *
 from networkx.generators.degree_seq import *
 from networkx.generators.directed import *
+from networkx.generators.duplication import *
 from networkx.generators.ego import *
 from networkx.generators.expanders import *
 from networkx.generators.geometric import *
+from networkx.generators.intersection import *
+from networkx.generators.joint_degree_seq import *
+from networkx.generators.lattice import *
 from networkx.generators.line import *
+from networkx.generators.mycielski import *
+from networkx.generators.nonisomorphic_trees import *
+from networkx.generators.random_clustered import *
 from networkx.generators.random_graphs import *
 from networkx.generators.small import *
-from networkx.generators.stochastic import *
 from networkx.generators.social import *
-from networkx.generators.threshold import *
-from networkx.generators.intersection import *
-from networkx.generators.random_clustered import *
-from networkx.generators.community import *
-from networkx.generators.nonisomorphic_trees import *
+from networkx.generators.stochastic import *
+from networkx.generators.trees import *
+from networkx.generators.triads import *
